/*------------------------------------------------------------------------------
* rtklib unit test driver : precise ephemeris function
*-----------------------------------------------------------------------------*/
#include <stdio.h>
#include <assert.h>
#include "../../src/rtklib.h"

static void dumpeph(peph_t *peph, int n)
{
	char s[64];
	int i,j;
	for (i=0;i<n;i++) {
		time2str(peph[i].time,s,3);
		printf("time=%s\n",s);
		for (j=0;j<MAXSAT;j++) {
			printf("%03d: %14.3f %14.3f %14.3f %14.3f : %5.3f %5.3f %5.3f %5.3f\n",
				   j+1,peph[i].pos[j][0],peph[i].pos[j][1],peph[i].pos[j][2],peph[i].pos[j][3]*1E9,
				   peph[i].std[j][0],peph[i].std[j][1],peph[i].std[j][2],peph[i].std[j][3]*1E9);
		}
	}
}
/* readsp3() */
void utest1(void)
{
	char *file1="igs14686.sp4";
	char *file2="igs14686.sp3";
	peph_t *eph1=NULL,*eph2=NULL;
	int n;
	
	n=readsp3(file1,&eph1);
		assert(n==0&&!eph1);
	n=readsp3(file2,&eph2);
		assert(n==96&&eph2);
	dumpeph(eph2,n);
	free(eph2);

	printf("%s utest1 : OK\n",__FILE__);
}
/* readsap() */
void utest2(void)
{
	double sap[MAXSAT][3]={{0}};
	double ep1[]={2008,3,1,0,0,0};
	double ep2[]={2006,11,4,23,59,59};
	char *file1="igs06.atx",*file2="..\\..\\data\\igs05.atx";
	gtime_t time;
	int i,stat;
	
	time=epoch2time(ep1);
	stat=readsap(file1,time,sap);
		assert(!stat);
	stat=readsap(file2,time,sap);
		assert(stat);
	for (i=0;i<MAXSAT;i++) {
		printf("PRN%02d : %7.4f %7.4f %7.4f\n",i+1,sap[i][0],sap[i][1],sap[i][2]);
	}
	printf("\n");
	time=epoch2time(ep2);
	stat=readsap(file2,time,sap);
		assert(stat);
	for (i=0;i<MAXSAT;i++) {
		printf("PRN%02d : %7.4f %7.4f %7.4f\n",i+1,sap[i][0],sap[i][1],sap[i][2]);
	}

	printf("%s utest2 : OK\n",__FILE__);
}
/* eph2posp() */
void utest3(void)
{
	FILE *fp;
	char *file="igs14686.sp3"; /* 2008/3/1 */
	peph_t *eph=NULL;
	int i,n,stat,sat;
	double ep[]={2008,3,1,0,0,0};
	double rs[3]={0},dts=0.0,tt;
	gtime_t t,time;
	
	n=readsp3(file,&eph);
		assert(n>0&&eph);
	time=epoch2time(ep);
	stat=eph2posp(time,0,eph,n,rs,&dts);
		assert(!stat);
	stat=eph2posp(time,160,eph,n,rs,&dts);
		assert(!stat);
	
	fp=fopen("testpeph.out","w");
	sat=3;
	for (i=0;i<86400;i+=30) {
		t=timeadd(time,(double)i);
		eph2posp(t,sat,eph,n,rs,&dts);
		tt=time2gpst(t,NULL);
		fprintf(fp,"%02d %6.0f %14.3f %14.3f %14.3f %14.3f\n",
			    sat,tt,rs[0],rs[1],rs[2],dts*1E9);
	}
	fclose(fp);
	printf("%s utest3 : OK\n",__FILE__);
}
int main(int argc, char **argv)
{
	utest1();
	utest2();
	utest3();
	return 0;
}
